#define MY_INFINITY 0xffffff00

__kernel void initialize(
	unsigned int vertex_cnt,
	__global unsigned int * d_graph_property)
{
    unsigned int tid = get_global_id(0);
    
    if ( tid < vertex_cnt )
    {
        d_graph_property[tid] = MY_INFINITY;
    }
}

__kernel void kernel_step(
	__global unsigned int * vlist,
	__global unsigned int * elist,
	unsigned int vertex_cnt,
	__global uint * vplist,
	uint curr,
	__global int * changed)
{
	unsigned int tid = get_global_id(0);
    
    if ( tid >= vertex_cnt ) return;

    if (vplist[tid]==curr)
    {
        uint start, end;
        start = vlist[tid];
        end = vlist[tid+1];
        for (uint i=start; i<end; i++)
        {
            uint vid = elist[i];
            if (vplist[vid]==MY_INFINITY)
            {
                *changed=true;
                vplist[vid]=curr+1;
            }
        }
    }
}

__kernel void kernel_step_OnTheFly(
	__global unsigned int * vlist,
	__global unsigned int * elist,
	unsigned int vertex_cnt,
	__global uint * vplist,
	uint curr,
	__global int * changed,
    __global int * remap)
{
	unsigned int tid = get_global_id(0);
    tid = remap[tid];
    
    if ( tid >= vertex_cnt ) return;

    if (vplist[tid]==curr)
    {
        uint start, end;
        start = vlist[tid];
        end = vlist[tid+1];
        for (uint i=start; i<end; i++)
        {
            uint vid = elist[i];
            if (vplist[vid]==MY_INFINITY)
            {
                *changed=true;
                vplist[vid]=curr+1;
            }
        }
    }
}

__kernel void kernel_step_profiler(
	__global unsigned int * vlist,
	__global unsigned int * elist,
	unsigned int vertex_cnt,
	__global uint * vplist,
	uint curr,
	__global int * changed,
    __global int * notify_size)
{
	unsigned int tid = get_global_id(0);
    
    if ( tid >= vertex_cnt ) return;

    if (vplist[tid]==curr)
    {
        uint start, end;
        start = vlist[tid];
        end = vlist[tid+1];

        notify_size[tid] = end - start;
        //for (uint i=start; i<end; i++)
        //{
        //    uint vid = elist[i];
        //    if (vplist[vid]==MY_INFINITY)
        //    {
        //        *changed=true;
        //        vplist[vid]=curr+1;
        //    }
        //}
    }
}

__kernel void kernel_step_MBAP(
	__global unsigned int * vlist,
	__global unsigned int * elist,
	unsigned int vertex_cnt,
	__global uint * vplist,
	uint curr,
	__global int * changed,
    __global int * remap,
    int alloc_low,
    int alloc_high)
{
	unsigned int tid = get_global_id(0);

    tid = remap[tid];

    int local_id = get_local_id(0);
    if (local_id < alloc_low || local_id >= alloc_high) {
        return;
    }

    if ( tid >= vertex_cnt ) return;

    if (vplist[tid]==curr)
    {
        uint start, end;
        start = vlist[tid];
        end = vlist[tid+1];

        for (uint i=start; i<end; i++)
        {
            uint vid = elist[i];
            if (vplist[vid]==MY_INFINITY)
            {
                *changed=true;
                vplist[vid]=curr+1;
            }
        }
    }
}
