#define MY_INFINITY 0xffffff00

__kernel void initialize(
	unsigned int vertex_cnt,
	__global unsigned int * d_vpl)
{
    unsigned int tid = get_global_id(0);
    
    if ( tid < vertex_cnt )
    {
        d_vpl[tid] = MY_INFINITY;
    }
}

__kernel void kernel_step(
	__global unsigned int * vlist,
	__global unsigned int * elist,
	unsigned int vertex_cnt,
	__global uint * vplist,
	__global uint * randlist,
	uint color,
	__global uint * device_over)
{
	int tid = get_global_id(0);
    
    if ( tid >= vertex_cnt ) return;
    
    if (vplist[tid]==MY_INFINITY)
    {
        uint vid = tid;
        uint start, end;
        start = vlist[vid];
        end = vlist[vid+1];
        
        uint local_rand = randlist[vid];
        bool found_larger=false;
        for (uint i=start; i<end; i++)
        {
            uint dest = elist[i];
            if (vplist[dest]<color) continue;
            if ( (randlist[dest]>local_rand) ||
                (randlist[dest]==local_rand && dest<vid))
            {
                found_larger = true;
                break;
            }
        }
        if (found_larger==false)
            vplist[vid] = color;
        else
            device_over[0] = false;
    }
}

__kernel void kernel_step_OnTheFly(
	__global unsigned int * vlist,
	__global unsigned int * elist,
	unsigned int vertex_cnt,
	__global uint * vplist,
	__global uint * randlist,
	uint color,
	__global uint * device_over,
    __global int * remap)
{
    int tid = get_global_id(0);
    tid = remap[tid];
    
    if ( tid >= vertex_cnt ) return;
    
    if (vplist[tid]==MY_INFINITY)
    {
        uint vid = tid;
        uint start, end;
        start = vlist[vid];
        end = vlist[vid+1];
        
        uint local_rand = randlist[vid];
        bool found_larger=false;
        for (uint i=start; i<end; i++)
        {
            uint dest = elist[i];
            if (vplist[dest]<color) continue;
            if ( (randlist[dest]>local_rand) ||
                (randlist[dest]==local_rand && dest<vid))
            {
                found_larger = true;
                break;
            }
        }
        if (found_larger==false)
            vplist[vid] = color;
        else
            device_over[0] = false;
    }
}

__kernel void kernel_step_profiler(
	__global unsigned int * vlist,
	__global unsigned int * elist,
	unsigned int vertex_cnt,
	__global uint * vplist,
	__global uint * randlist,
	uint color,
	__global uint * device_over,
    __global int* notify_size)
{
	int tid = get_global_id(0);
    
    if ( tid >= vertex_cnt ) return;
    
    if (vplist[tid]==MY_INFINITY)
    {
        uint vid = tid;
        uint start, end;
        start = vlist[vid];
        end = vlist[vid+1];

        //uint local_rand = randlist[vid];
        //bool found_larger=false;

        notify_size[tid] = end - start;

        //for (uint i=start; i<end; i++)
        //{
        //    uint dest = elist[i];
        //    if (vplist[dest]<color) continue;
        //    if ( (randlist[dest]>local_rand) ||
        //        (randlist[dest]==local_rand && dest<vid))
        //    {
        //        found_larger = true;
        //        break;
        //    }
        //}
        //if (found_larger==false)
        //    vplist[vid] = color;
        //else
        //    *device_over = false;
    }
}

__kernel void kernel_step_MBAP(
    __global unsigned int * vlist,
    __global unsigned int * elist,
    unsigned int vertex_cnt,
    __global uint * vplist,
    __global uint * randlist,
    uint color,
    __global uint * device_over,
    __global int *remap,
    int alloc_low,
    int alloc_high)
{
    unsigned int tid = get_global_id(0);
    tid = remap[tid];

    int local_id = get_local_id(0);
    if (local_id < alloc_low || local_id >= alloc_high) {
        return;
    }

    if ( tid >= vertex_cnt ) return;
    
    if (vplist[tid]==MY_INFINITY)
    {
        uint vid = tid;
        uint start, end;
        start = vlist[vid];
        end = vlist[vid+1];
        
        uint local_rand = randlist[vid];
        uint found_larger=0;
        for (uint i=start; i<end; i++)
        {
            uint dest = elist[i];
            if (vplist[dest]<color) continue;
            if ( (randlist[dest]>local_rand) ||
                (randlist[dest]==local_rand && dest<vid))
            {
                found_larger = 1;
                break;
            }
        }
        if (found_larger==0)
            vplist[vid] = color;
        else
            *device_over = 0;
    }
}

