#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

// first index: total number of CPU cores of the system
// second index: the number of arguments for this kernel
unsigned long x86_kernel_params[48] = {
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
};

int num_total_cores = 4;
int num_total_params = 12;

// this interaction should be changed later
int num_proper_cpu_cores = 4;
int num_proper_gpu_cores = 64;

#define MY_INFINITY 0xffffff00

void kernel_step_CPU(unsigned int *vlist, unsigned int *elist, unsigned int vertex_cnt,
        unsigned int *vplist, unsigned int *randlist, unsigned int color, unsigned int *device_over,
        int* remap, size_t start_wg_id, size_t wg_size, size_t num_wgs)
{
    for (size_t wg_id = start_wg_id; wg_id < start_wg_id + num_wgs; wg_id++) {
        size_t wi_id = wg_id * wg_size;

        for (size_t global_id = wi_id; global_id < wi_id + wg_size; global_id++) {
            int tid = (int)global_id; // get_global_id(0);
            tid = remap[tid];

            if ( tid >= vertex_cnt ) continue;

            if (vplist[tid]==MY_INFINITY)
            {
                uint vid = tid;
                uint start, end;
                start = vlist[vid];
                end = vlist[vid+1];

                uint local_rand = randlist[vid];
                uint found_larger=0;
                for (uint i=start; i<end; i++)
                {
                    uint dest = elist[i];
                    if (vplist[dest]<color) continue;
                    if ( (randlist[dest]>local_rand) ||
                            (randlist[dest]==local_rand && dest<vid))
                    {
                        found_larger = 1;
                        break;
                    }
                }
                if (found_larger==0)
                    vplist[vid] = color;
                else
                    *device_over = 0;
            }
        }
    }
}

extern "C" {
    int CPU_thread_entry(int id) {
        unsigned int* vlist = (unsigned int*)x86_kernel_params[num_total_params * id + 0];
        unsigned int* elist = (unsigned int*)x86_kernel_params[num_total_params * id + 1];
        unsigned int* vertex_cnt = (unsigned int*)x86_kernel_params[num_total_params * id + 2];
        unsigned int* vplist = (unsigned int*)x86_kernel_params[num_total_params * id + 3];
        unsigned int* randlist = (unsigned int*)x86_kernel_params[num_total_params * id + 4];
        unsigned int* color = (unsigned int*)x86_kernel_params[num_total_params * id + 5];
        unsigned int* device_over = (unsigned int*)x86_kernel_params[num_total_params * id + 6];
        int* remap = (int*)x86_kernel_params[num_total_params * id + 7];
        size_t start_wg_id = (size_t)x86_kernel_params[num_total_params * id + 8];
        size_t wg_size = (size_t)x86_kernel_params[num_total_params * id + 9];
        size_t num_wgs = (size_t)x86_kernel_params[num_total_params * id + 10];

        //printf("[CPU thread] id: %d\n", id);

        //printf("vlist: %lu value: %d\n", vlist, *vlist);
        //printf("device_over: %lu value: %d\n", device_over, *device_over);
        //printf("vertex_cnt: %d\n", *vertex_cnt);

        kernel_step_CPU(vlist, elist, *vertex_cnt,
                vplist, randlist, *color, device_over,
                remap, start_wg_id, wg_size, num_wgs);

        int* has_finished = (int*)x86_kernel_params[num_total_params * id + 11];
        *has_finished += 1;

        return 0;
    }
}
