#!/usr/bin/env python

import numpy as np
from os.path import exists
import os
import sys

import re

number_re = "+([0-9]+\.[01-9]+|[0-9]+)"

kernel_index = {"fullchip":1,"scale20":1,"in-2004":0,"Watson_Gene":1,"LDBC":0,"Knowledge_Repo":1}

graphs = ["fullchip","scale20","in-2004","Watson_Gene","LDBC","Knowledge_Repo"]

def mean(data):
    n = len(data)
    return sum(data)/n

def _ss(data):
    c = mean(data)
    ss = sum((x-c)**2 for x in data)
    return ss

def stddev(data):
    n = len(data)
    ss = _ss(data)
    pvar = ss/n
    return pvar**0.5

def compute_geomean(\
    values):
    a = np.array(values)
    return a.prod()**(1.0/len(a))

graph_id = { "fullchip": 1,\
             "scale20": 2,\
             "in-2004": 3,\
             "Watson_Gene": 4,\
             "LDBC": 5,\
             "Knowledge_Repo": 6 }

graph_name = { "fullchip": "fullchip",\
               "scale20": "scale20",\
               "in-2004": "in-2004",\
               "Watson_Gene": "WatsonGene",\
               "LDBC": "LDBC",\
               "Knowledge_Repo": "KnowledgeRepo" }

cpu_absolute_time = { "fullchip": 0,\
                      "scale20": 0,\
                      "in-2004": 0,\
                      "Watson_Gene": 0,\
                      "LDBC": 0,\
                      "Knowledge_Repo": 0 }

cpu_num_kernels = { "fullchip": 0,\
                    "scale20": 0,\
                    "in-2004": 0,\
                    "Watson_Gene": 0,\
                    "LDBC": 0,\
                    "Knowledge_Repo": 0 }

gpu_absolute_time = { "fullchip": 0,\
                      "scale20": 0,\
                      "in-2004": 0,\
                      "Watson_Gene": 0,\
                      "LDBC": 0,\
                      "Knowledge_Repo": 0 }

gpu_num_kernels = { "fullchip": 0,\
                    "scale20": 0,\
                    "in-2004": 0,\
                    "Watson_Gene": 0,\
                    "LDBC": 0,\
                    "Knowledge_Repo": 0 }

normalization = { "fullchip": 0,\
                  "scale20": 0,\
                  "in-2004": 0,\
                  "Watson_Gene": 0,\
                  "LDBC": 0,\
                  "Knowledge_Repo": 0 }

csv_file = open("./gpu_absolute.csv", "w")

for i in [1, 2, 3]:
    f = open("./csv/gpu_run_"+str(i)+".csv", "r")
    line = f.readline() # ignore the first line
    line = f.readline()
    while line:
        elems = line.split(",")
        #print elems
        gpu_absolute_time[elems[0]] += float(elems[kernel_index[elems[0]] + 3])
        line = f.readline()

for graph in graphs:
    gpu_absolute_time[graph] /= 3

    csv_file.write(str(graph_id[graph]) + "," + graph_name[graph] + "," + str(gpu_absolute_time[graph]) + "," + "0.0" + "," + "0.0" + "," + "0.0" + "\n")

csv_file.close()

csv_file = open("./cpu_absolute.csv", "w")

for i in [1, 2, 3]:
    f = open("./csv/cpu_run_"+str(i)+".csv", "r")
    line = f.readline() # ignore the first line
    line = f.readline()
    while line:
        elems = line.split(",")
        #print elems
        cpu_absolute_time[elems[0]] += float(elems[kernel_index[elems[0]] + 3])
        line = f.readline()

for graph in graphs:
    cpu_absolute_time[graph] /= 3

    csv_file.write(str(graph_id[graph]) +"," + graph_name[graph] + "," + str(cpu_absolute_time[graph]) + "," + "0.0" + "," + "0.0" + "," + "0.0" + "\n")

csv_file.close()

for graph in graphs:
    normalization[graph] = cpu_absolute_time[graph]

# GPU run
csv_file = open("./gpu_normalized.csv", "w")
csv_file.write("#ID,graph,normalized execution time\n")

gpu_kernel_time_arr = []
for graph in graphs:
    gpu_normalized_time = gpu_absolute_time[graph] / normalization[graph]
    gpu_kernel_time_arr.append(gpu_normalized_time)
    if gpu_normalized_time > 4.0:
        csv_file.write(str(graph_id[graph]) + "," + graph_name[graph] + "," + str(gpu_normalized_time) + "," + "0.0" + "," + "0.0" + "," + "0.0" + ",4," + str(round(gpu_normalized_time,1)) + "\n")
    else:
        csv_file.write(str(graph_id[graph]) + "," + graph_name[graph] + "," + str(gpu_normalized_time) + "," + "0.0" + "," + "0.0" + "," + "0.0" + "," + "\n")
gpu_kernel_time_geomean = compute_geomean(gpu_kernel_time_arr)
if (gpu_kernel_time_geomean > 4.0):
    csv_file.write("7" + "," + "Geomean" + "," + str(gpu_kernel_time_geomean) + "," + "0.0" + "," + "0.0" + "," + "0.0" + ",4," + str(round(gpu_kernel_time_geomean,1)) + "\n")
else:
    csv_file.write("7" + "," + "Geomean" + "," + str(gpu_kernel_time_geomean) + "," + "0.0" + "," + "0.0" + "," + "0.0" + "," + "\n")

csv_file.close()

csv_file = open("./cpu_normalized.csv", "w")
csv_file.write("#ID,graph,normalized execution time\n")

cpu_kernel_time_arr = []
for graph in graphs:
    cpu_normalized_time = cpu_absolute_time[graph] / normalization[graph]
    cpu_kernel_time_arr.append(cpu_normalized_time)
    if cpu_normalized_time > 4.0:
        csv_file.write(str(graph_id[graph]) + "," + graph_name[graph] + "," + str(cpu_normalized_time) + "," + "0.0" + "," + "0.0" + "," + "0.0" + ",4," + str(round(cpu_normalized_time,1)) + "\n")
    else:
        csv_file.write(str(graph_id[graph]) + "," + graph_name[graph] + "," + str(cpu_normalized_time) + "," + "0.0" + "," + "0.0" + "," + "0.0" + "," + "\n")
cpu_kernel_time_geomean = compute_geomean(cpu_kernel_time_arr)
if (cpu_kernel_time_geomean > 4.0):
    csv_file.write("7" + "," + "Geomean" + "," + str(cpu_kernel_time_geomean) + "," + "0.0" + "," + "0.0" + "," + "0.0" + ",4," + str(round(cpu_kernel_time_geomean,1)) + "\n")
else:
    csv_file.write("7" + "," + "Geomean" + "," + str(cpu_kernel_time_geomean) + "," + "0.0" + "," + "0.0" + "," + "0.0" + "," + "\n")

csv_file.close()

csv_file = open("./workgroup_normalized.csv", "w")
csv_file.write("#ID,graph,normalized execution time,,,best CPU ratio,\n")

kernel_time_arr = []
for graph in graphs:
    best_kernel_time = 1000000000
    best_cpu_ratio = 0
    for cpu_ratio in range(5, 100, 5):
        f = open("./csv/workgroup_partitioning/"+graph+"."+str(cpu_ratio)+"."+str(100-cpu_ratio)+".1", "r")
        line = f.readline() # ignore the first line
        line = f.readline()
        elems = line.split(",")
        if float(elems[kernel_index[graph]+2]) < best_kernel_time:
            best_kernel_time = float(elems[kernel_index[graph]+2])
            best_cpu_ratio = cpu_ratio
        f.close()
    best_kernel_time = best_kernel_time / normalization[graph]
    kernel_time_arr.append(best_kernel_time)
    if best_kernel_time > 4.0:
        csv_file.write(str(graph_id[graph]) + "," + graph_name[graph] + "," + str(best_kernel_time) + "," + "0.0" + "," + "0.0" + "," + str(best_cpu_ratio) + ",4," + str(round(best_kernel_time,1)) + "\n")
    else:
        csv_file.write(str(graph_id[graph]) +"," + graph_name[graph] + "," + str(best_kernel_time) + "," + "0.0" + "," + "0.0" + "," + str(best_cpu_ratio) + "," + "\n")
kernel_time_geomean = compute_geomean(kernel_time_arr)
if (kernel_time_geomean > 4.0):
    csv_file.write("7" + "," + "Geomean" + "," + str(kernel_time_geomean) + "," + "0.0" + "," + "0.0" + "," + "0.0" + ",4," + str(round(kernel_time_geomean,1)) + "\n")
else:
    csv_file.write("7" + "," + "Geomean" + "," + str(kernel_time_geomean) + "," + "0.0" + "," + "0.0" + "," + "0.0" + "," + "\n")

csv_file.close()

csv_file = open("./oracle_normalized.csv", "w")
csv_file.write("#ID,graph,normalized execution time,profiler overhead,reshaping overhead,best CPU ratio,\n")

kernel_time_arr = []
profiler_overhead_arr = []
reshaping_overhead_arr = []
for graph in graphs:
    best_first_kernel_time = 1000000000
    best_profiler_overhead = 0
    best_finepar_overhead = 0
    best_kernel_time = 0
    best_cpu_ratio = 0
    for cpu_ratio in range(1, 100, 1):
        f = open("./csv/oracle/"+graph+"."+str(cpu_ratio)+"."+str(100-cpu_ratio)+".1", "r")
        line = f.readline() # ignore the first line
        line = f.readline()
        elems = line.split(",")
        if float(elems[kernel_index[graph]+5]) < best_first_kernel_time:
            best_first_kernel_time = float(elems[kernel_index[graph]+5]) ## if elems[3] <- global optimal
            best_kernel_time = float(elems[kernel_index[graph]+5]) ## if elems[3] <- global optimal
            best_profiler_overhead = float(elems[0])
            best_finepar_overhead = float(elems[2])
            best_cpu_ratio = cpu_ratio
        f.close()
    best_kernel_time = best_kernel_time / normalization[graph]
    best_profiler_overhead = best_profiler_overhead / normalization[graph]
    best_finepar_overhead = best_finepar_overhead / normalization[graph]
    kernel_time_arr.append(best_kernel_time)
    profiler_overhead_arr.append(best_profiler_overhead)
    reshaping_overhead_arr.append(best_finepar_overhead)
    if (best_kernel_time+best_profiler_overhead+best_finepar_overhead) > 4.0:
        csv_file.write(str(graph_id[graph]) + "," + graph_name[graph] + "," + str(best_kernel_time) + "," + str(best_profiler_overhead) + "," + str(best_finepar_overhead) + "," + str(best_cpu_ratio) + ",4," + str(round(best_kernel_time+best_profiler_overhead+best_finepar_overhead,1)) + "," + str(best_kernel_time+best_profiler_overhead+best_finepar_overhead) + "\n")
    else:
        csv_file.write(str(graph_id[graph]) + "," + graph_name[graph] + "," + str(best_kernel_time) + "," + str(best_profiler_overhead) + "," + str(best_finepar_overhead) + "," + str(best_cpu_ratio) + ",,," + str(best_kernel_time+best_profiler_overhead+best_finepar_overhead)+"\n")
kernel_time_geomean = compute_geomean(kernel_time_arr)
profiler_overhead_geomean = compute_geomean(profiler_overhead_arr)
reshaping_overhead_geomean = compute_geomean(reshaping_overhead_arr)
if (kernel_time_geomean+profiler_overhead_geomean+reshaping_overhead_geomean)> 4.0:
    csv_file.write("7" + "," + "Geomean" + "," + str(kernel_time_geomean) + "," + str(profiler_overhead_geomean) + "," + str(reshaping_overhead_geomean) + "," + ",4," + str(round(kernel_time_geomean+profiler_overhead_geomean+reshaping_overhead_geomean,1)) + "," + str(kernel_time_geomean+profiler_overhead_geomean+reshaping_overhead_geomean) + "\n")
else:
    csv_file.write("7" + "," + "Geomean" + "," + str(kernel_time_geomean) + "," + str(profiler_overhead_geomean) + "," + str(reshaping_overhead_geomean) + "," + ",,," + str(kernel_time_geomean+profiler_overhead_geomean+reshaping_overhead_geomean) + "\n")

csv_file.close()

csv_file = open("./on_the_fly_normalized.csv", "w")
csv_file.write("#ID,graph,normalized execution time,profiler overhead,reshaping overhead,\n")

kernel_time_arr = []
profiler_overhead_arr = []
reshaping_overhead_arr = []
std_arr = []
for graph in graphs:
    profiler_overhead = 0
    reshaping_overhead = 0
    kernel_time = 0
    total_time_arr = []

    for i in [1,2,3]:
        f = open("./csv/on_the_fly/"+graph+"."+str(i)+".csv", "r")
        line = f.readline() # ignore the first line
        line = f.readline()
        elems = line.split(",")
        kernel_time += float(elems[3*kernel_index[graph]+5]) ## if elems[3] <- global optimal
        profiler_overhead += float(elems[3*kernel_index[graph]+6])
        reshaping_overhead += float(elems[3*kernel_index[graph]+7])
        total_time_arr.append((float(elems[3*kernel_index[graph]+5])+ float(elems[3*kernel_index[graph]+6]) + float(elems[3*kernel_index[graph]+7])) / normalization[graph])
        f.close()

    kernel_time /= 3
    profiler_overhead /= 3
    reshaping_overhead /= 3

    kernel_time = kernel_time / normalization[graph]
    profiler_overhead = profiler_overhead / normalization[graph]
    reshaping_overhead = reshaping_overhead / normalization[graph]
    kernel_time_arr.append(kernel_time)
    profiler_overhead_arr.append(profiler_overhead)
    reshaping_overhead_arr.append(reshaping_overhead)
    std = stddev(total_time_arr)
    std_arr.append(std)
    if (kernel_time+profiler_overhead+reshaping_overhead) > 4.0:
        csv_file.write(str(graph_id[graph]) + "," + graph_name[graph] + "," + str(kernel_time) + "," + str(profiler_overhead) + "," + str(reshaping_overhead) + ",4," + str(round(kernel_time+profiler_overhead+reshaping_overhead,1)) + "," + str(kernel_time+profiler_overhead+reshaping_overhead)+"\n")
    else:
        csv_file.write(str(graph_id[graph]) + "," + graph_name[graph] + "," + str(kernel_time) + "," + str(profiler_overhead) + "," + str(reshaping_overhead) + ",,," + str(kernel_time+profiler_overhead+reshaping_overhead) + "\n")
kernel_time_geomean = compute_geomean(kernel_time_arr)
profiler_overhead_geomean = compute_geomean(profiler_overhead_arr)
reshaping_overhead_geomean = compute_geomean(reshaping_overhead_arr)
std_geomean = mean(std_arr)
if (kernel_time_geomean+profiler_overhead_geomean+reshaping_overhead_geomean) > 4.0:
    csv_file.write("7" + "," + "Geomean" + "," + str(kernel_time_geomean) + "," + str(profiler_overhead_geomean) + "," + str(reshaping_overhead_geomean) + "," + ",4," + str(round(kernel_time_geomean+profiler_overhead_geomean+reshaping_overhead_geomean),1) + "," + str(kernel_time_geomean+profiler_overhead_geomean+reshaping_overhead_geomean) + "," + str(std_geomean) + "\n")
else:
    csv_file.write("7" + "," + "Geomean" + "," + str(kernel_time_geomean) + "," + str(profiler_overhead_geomean) + "," + str(reshaping_overhead_geomean) + "," + ",," + str(kernel_time_geomean + profiler_overhead_geomean + reshaping_overhead_geomean) + "," + str(std_geomean) + "\n")

csv_file.close()

