__kernel void PageRank(__global int *rowstr,
                       __global float *p,
                       __global int *colidx,
                       __global float *q,
                       int numPages)
{
    int id = get_global_id(0);

    if (id >= numPages) return;

    float DAM = 0.85;

    float newRank = 0.0;
    //#pragma irregular
    for (int k = rowstr[id]; k < rowstr[id+1]; k++) {
        newRank += p[colidx[k]];
    }

    newRank = (1.0 - DAM) / numPages + DAM * newRank;

    q[id] = newRank / numPages;
}

__kernel void PageRank_profiler(__global int *rowstr,
                                __global float *p,
                                __global int *colidx,
                                __global float *q,
                                int numPages,
                                __global int* notify_size)
{
    int id = get_global_id(0);

    notify_size[id] = rowstr[id+1] - rowstr[id];
}

__kernel void PageRank_OnTheFly(__global int *rowstr,
                                __global float *p,
                                __global int *colidx,
                                __global float *q,
                                int numPages,
                                __global int* remap)
{
    int id = get_global_id(0);
    id = remap[id];

    if (id >= numPages) return;

    float DAM = 0.85;

    float newRank = 0.0;
    //#pragma irregular
    for (int k = rowstr[id]; k < rowstr[id+1]; k++) {
        newRank += p[colidx[k]];
    }

    newRank = (1.0 - DAM) / numPages + DAM * newRank;

    q[id] = newRank / numPages;
}


__kernel void PageRank_MBAP(__global int *rowstr,
                            __global float *p,
                            __global int *colidx,
                            __global float *q,
                            int numPages,
                            __global int* remap,
                            int alloc_low,
                            int alloc_high)
{
    int id = get_global_id(0);
    id = remap[id];

    //int local_id = get_local_id(0);
    //if (local_id < alloc_low || local_id >= alloc_high) {
    //    return;
    //}

    if (id >= numPages) return;

    float DAM = 0.85;

    float newRank = 0.0;
    //#pragma irregular
    for (int k = rowstr[id]; k < rowstr[id+1]; k++) {
        newRank += p[colidx[k]];
    }

    newRank = (1.0 - DAM) / numPages + DAM * newRank;

    q[id] = newRank / numPages;
}

