#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

// first index: total number of CPU cores of the system
// second index: the number of arguments for this kernel
unsigned long x86_kernel_params[44] = {
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
    3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
};

int num_total_cores = 4;
int num_total_params = 11;

// this interaction should be changed later
int num_proper_cpu_cores = 4;
int num_proper_gpu_cores = 0;

void SpMV_CPU(int* rowstr, float* a, float* p, int* colidx, float* q, int numRows,
        int* remap, size_t start_wg_id, size_t wg_size, size_t num_wgs)
{
    for (size_t wg_id = start_wg_id; wg_id < start_wg_id + num_wgs; wg_id++) {
        size_t wi_id = wg_id * wg_size;

        for (size_t global_id = wi_id; global_id < wi_id + wg_size; global_id++) {
            int id = (int)global_id; // get_global_id(0);
            id = remap[id];

            //printf("id: %d remap: %d size: %d\n", global_id, id, rowstr[id+1]-rowstr[id]);

            if (id < numRows) {
                float suml = 0.0;
                for (int k = rowstr[id]; k < rowstr[id+1]; k++) {
                    suml = suml + a[k]*p[colidx[k]];
                }

                q[id] = suml;
            }
        }
    }
}

extern "C" {
int CPU_thread_entry(int id) {
    int* rowstr = (int*)x86_kernel_params[num_total_params * id + 0];
    float* a = (float*)x86_kernel_params[num_total_params * id + 1];
    float* p = (float*)x86_kernel_params[num_total_params * id + 2];
    int* colidx = (int*)x86_kernel_params[num_total_params * id + 3];
    float* q = (float*)x86_kernel_params[num_total_params * id + 4];
    int* numRows = (int*)x86_kernel_params[num_total_params * id + 5];
    int* remap = (int*)x86_kernel_params[num_total_params * id + 6];
    size_t start_wg_id = (size_t)x86_kernel_params[num_total_params * id + 7];
    size_t wg_size = (size_t)x86_kernel_params[num_total_params * id + 8];
    size_t num_wgs = (size_t)x86_kernel_params[num_total_params * id + 9];

    //printf("[CPU thread] id: %d start_wg_id: %lu (%d) wg_size: %lu num_wgs: %lu\n", id, start_wg_id, start_wg_id * wg_size, wg_size, num_wgs);

    SpMV_CPU(rowstr, a, p, colidx, q, *numRows, remap, start_wg_id, wg_size, num_wgs);

    int* has_finished = (int*)x86_kernel_params[num_total_params * id + 10];
    *has_finished += 1;

    return 0;
}
}
