__kernel void SpMV(__global int *rowstr,
                   __global float *a,
                   __global float *p,
                   __global int *colidx,
                   __global float *q,
                   int numRows)
{
    int id = get_global_id(0);

    if (id < numRows) {
        float suml = 0.0;
        //#pragma irregular
        for (int k = rowstr[id]; k < rowstr[id+1]; k++) {
            suml = mad(a[k], p[colidx[k]], suml);
        }

        q[id] = suml;
    }
}

__kernel void SpMV_OnTheFly(__global int *rowstr,
                   __global float *a,
                   __global float *p,
                   __global int *colidx,
                   __global float *q,
                   int numRows,
                   __global int * remap)
{
    int id = get_global_id(0);
    id = remap[id];

    if (id < numRows) {
        float suml = 0.0;
        //#pragma irregular
        for (int k = rowstr[id]; k < rowstr[id+1]; k++) {
            suml = mad(a[k], p[colidx[k]], suml);
        }

        q[id] = suml;
    }
}

__kernel void SpMV_profiler(__global int *rowstr,
                   __global float *a,
                   __global float *p,
                   __global int *colidx,
                   __global float *q,
                   int numRows,
                   __global int* notify_size)
{
    int id = get_global_id(0);

    if (id < numRows) {
        notify_size[id] = rowstr[id+1] - rowstr[id];
    }
}

__kernel void SpMV_MBAP(__global int *rowstr,
                   __global float *a,
                   __global float *p,
                   __global int *colidx,
                   __global float *q,
                   int numRows,
                   __global int* remap,
                   int alloc_low,
                   int alloc_high)
{
    int id = get_global_id(0);
    id = remap[id];

    if (id < numRows) {
        //int local_id = get_local_id(0);
        //if (local_id < alloc_low || local_id >= alloc_high) {
        //    return;
        //}

        float suml = 0.0;
        for (int k = rowstr[id]; k < rowstr[id+1]; k++) {
            suml = mad(a[k], p[colidx[k]], suml);
        }

        q[id] = suml;
    }
}

