//------------------------------------------------------------------------------
// ARM OpenCL Runtime
//
// Copyright (c) Computer Systems and Platforms Laboratory
// http://csap.snu.ac.kr
// All right reserved.
//
// Author:
// Younghyun Cho <younghyun@csap.snu.ac.kr>
//

#ifndef __ARM_OpenCL_Command_Queue_H_
#define __ARM_OpenCL_Command_Queue_H_

#include <CL/cl.h>
#include "ArmCLContext.h"
#include "ArmCLKernel.h"

#include <deque>
//#include <pthread.h>
#include <semaphore.h>
#include <thread>

class ArmCLCommandQueue {
  public:
    ArmCLCommandQueue(cl_command_queue command_queue,
            ArmCLContext* armcl_context);
    ~ArmCLCommandQueue();

    cl_command_queue command_queue() const { return command_queue_; }
    cl_command_queue cpu_command_queue() const { return cpu_command_queue_; }

    cl_int CPUThreading(ArmCLKernel* armcl_kernel,
            cl_uint work_dim,
            const size_t* global_work_offset,
            const size_t* global_work_size,
            const size_t* local_work_size,
            cl_uint num_events_in_wait_list,
            const cl_event* event_wait_list,
            cl_event* event);

    cl_int WorkGroupPartitioning(ArmCLKernel* armcl_kernel,
            cl_uint work_dim,
            const size_t* global_work_offset,
            const size_t* global_work_size,
            const size_t* local_work_size,
            cl_uint num_events_in_wait_list,
            const cl_event* event_wait_list,
            cl_event* event);

    cl_int OnTheFly(ArmCLKernel* armcl_kernel,
            cl_uint work_dim,
            const size_t* global_work_offset,
            const size_t* global_work_size,
            const size_t* local_work_size,
            cl_uint num_events_in_wait_list,
            const cl_event* event_wait_list,
            cl_event* event);

    cl_int Manual_Static(ArmCLKernel* armcl_kernel,
            cl_uint work_dim,
            const size_t* global_work_offset,
            const size_t* global_work_size,
            const size_t* local_work_size,
            cl_uint num_events_in_wait_list,
            const cl_event* event_wait_list,
            cl_event* event);

  private:
    void PrepareOpenCLCalls();

    // prepare remap buffers in advance
    int* in_order_map_;
    int* remap_;
    int* array_gpu_;
    int* array_cpu_;
    int* loop_size_;
    const int max_work_items = 10000000;

    cl_context context_;
    cl_context cpu_context_;

    cl_device_id device_;
    cl_device_id cpu_device_;

    cl_uint num_gpu_compute_units_;
    cl_uint num_cpu_compute_units_;

    cl_command_queue command_queue_;
    cl_command_queue cpu_command_queue_;

    ArmCLContext* armcl_context_;

    cl_command_queue CL_API_CALL (*clCreateCommandQueue_p_) (
            cl_context context,
            cl_device_id device,
            cl_command_queue_properties properties,
            cl_int* errcode_ret);

    cl_int CL_API_CALL (*clEnqueueNDRangeKernel_p_) (
            cl_command_queue command_queue,
            cl_kernel kernel,
            cl_uint work_dim,
            const size_t* global_work_offset,
            const size_t* global_work_size,
            const size_t* local_work_size,
            cl_uint num_events_in_wait_list,
            const cl_event* event_wait_list,
            cl_event* event);

    cl_int CL_API_CALL (*clSetKernelArg_p_)(cl_kernel kernel,
            cl_uint arg_index,
            size_t arg_size,
            const void* arg_value);

    cl_mem CL_API_CALL (*clCreateBuffer_p_)(cl_context context,
            cl_mem_flags flags,
            size_t size,
            void* host_ptr,
            cl_int* errcode_ret);

    cl_int CL_API_CALL (*clEnqueueWriteBuffer_p_)(
            cl_command_queue command_queue,
            cl_mem buffer,
            cl_bool blocking_write,
            size_t offset,
            size_t size,
            const void* ptr,
            cl_uint num_events_in_wait_list,
            const cl_event* event_wait_list,
            cl_event* event);

    cl_int CL_API_CALL (*clGetEventInfo_p_)(
            cl_event event,
            cl_profiling_info param_name,
            size_t param_value_size,
            void* param_value,
            size_t* param_value_size_ret);
};

#endif

