//------------------------------------------------------------------------------
// ARM OpenCL Runtime
//
// Copyright (c) Computer Systems and Platforms Laboratory
// http://csap.snu.ac.kr
// All right reserved.
//
// Author:
// Younghyun Cho <younghyun@csap.snu.ac.kr>
//

#ifndef __ARM_OpenCL_Context_H_
#define __ARM_OpenCL_Context_H_

#include <CL/cl.h>
#include <cstdlib>
#include <iostream>
#include <string>
#include <iostream>
#include <dlfcn.h>
#include <stdio.h>

#include <vector>

class ArmCLProgram;
class ArmCLKernel;

class ArmCLProgramSource {
  public:
    ArmCLProgramSource() {}
    ~ArmCLProgramSource() {}

    const char* concat_source() const { return concat_source_.c_str(); }
    size_t concat_source_length() const { return concat_source_.length(); }
    const char* header_name() const { return header_name_.c_str(); }

    void AddSource(const char* source);
    void AddSource(const char* source, size_t length);
    void SetHeaderName(const char* header_name);

  private:
    std::vector<std::string> sources_;
    std::string concat_source_;
    std::string header_name_;
};

class ArmCLContext {
  public:
    ArmCLContext(cl_context context);
    ~ArmCLContext();

    cl_context context() const { return context_; }
    cl_context cpu_context() const { return cpu_context_; }

    cl_device_id gpu_device_id() const { return gpu_device_; }
    cl_device_id cpu_device_id() const { return cpu_device_; }

    cl_uint num_gpu_compute_units() const { return num_gpu_compute_units_; }
    cl_uint num_cpu_compute_units() const { return num_cpu_compute_units_; }

    void RegisterPrograms(ArmCLProgram* armcl_program);

    void PreprocessingWithSource(
            cl_uint count,
            const char ** strings,
            const size_t * lengths);

  private:
    void PrepareOpenCLCalls();
    void PrepareCPUContext();
    void QueryDeviceInfo();

    std::vector<ArmCLProgram*> armcl_programs_;

    cl_context context_;
    cl_context cpu_context_;

    cl_device_id gpu_device_;
    cl_device_id cpu_device_;

    cl_uint num_gpu_compute_units_;
    cl_uint num_cpu_compute_units_;

    ArmCLProgramSource* source_;

    cl_context CL_API_CALL (*clCreateContext_p_) (
            const cl_context_properties* properties,
            cl_uint num_devices,
            const cl_device_id* devices,
            void (*pfn_notify) (
                const char* effinfo,
                const void* private_info,
                size_t cb,
                void* user_data
                ),
            void* user_data,
            cl_int* errcode_ret);
};

#endif
