//------------------------------------------------------------------------------
// ARM OpenCL Runtime
//
// Copyright (c) Computer Systems and Platforms Laboratory
// http://csap.snu.ac.kr
// All right reserved.
//
// Author:
// Younghyun Cho <younghyun@csap.snu.ac.kr>
//

#include "ArmCLProgram.h"

ArmCLProgram::ArmCLProgram(cl_program program, cl_program cpu_program,
        ArmCLContext* armcl_context)
    : program_(program), cpu_program_(cpu_program),
    armcl_context_(armcl_context)
{
#ifdef DEBUG
    std::cout << "ArmCL: "
        << " program created"
        << std::endl;
#endif

    PrepareOpenCLCalls();
}

ArmCLProgram::~ArmCLProgram()
{
#ifdef DEBUG
    std::cout << "ArmCL: "
        << " program released"
        << std::endl;
#endif
}

void ArmCLProgram::BuildCPUProgram()
{
    cl_device_id cpu_device = armcl_context_->cpu_device_id();
    cl_int err;

    err = clBuildProgram_p_(cpu_program_, // program
            1, // num_devices
            &cpu_device, // device id
            NULL, // options
            NULL, // callback function
            NULL); // user data
    if (err != CL_SUCCESS) {
        std::cout << "ArmCL: "
            << "clBuildProgram failed for cpu program" << std::endl;
    }
}

void ArmCLProgram::RegisterKernels(ArmCLKernel* armcl_kernel)
{
    armcl_kernels_.push_back(armcl_kernel);
}

void ArmCLProgram::PreprocessingWithSource(const char ** strings,
        const size_t * lengths,
        char ** generated_strings,
        size_t * generated_lengths)
{
#ifdef DEBUG
    std::cout << "ArmCL: "
        << " preprocessing with source"
        << std::endl;
#endif
}

void ArmCLProgram::PrepareOpenCLCalls()
{
    char* error;

    clBuildProgram_p_ = (cl_int CL_API_CALL (*) (cl_program program,
                cl_uint num_devices,
                const cl_device_id* device_list,
                const char* options,
                void (CL_CALLBACK * pfn_notify)(cl_program program,
                    void* user_data),
                void* user_data))dlsym(RTLD_NEXT, "clBuildProgram");
    if ((error = dlerror()) != NULL) {
        fputs(error, stderr);
        exit(1);
    }
}
