//------------------------------------------------------------------------------
// ARM OpenCL Runtime
//
// Copyright (c) Computer Systems and Platforms Laboratory
// http://csap.snu.ac.kr
// All right reserved.
//
// Author:
// Younghyun Cho <younghyun@csap.snu.ac.kr>
//

#ifndef __ARM_OpenCL_Program_H_
#define __ARM_OpenCL_Program_H_

#include <CL/cl.h>
#include <iostream>
#include <string>
#include <vector>
#include <stdio.h>
#include <cstdlib>

#include <dlfcn.h>
#include <stdio.h>

//#include "ArmCLKernel.h"
#include "ArmCLContext.h"

class ArmCLContext;
class ArmCLKernel;

class ArmCLProgram {
  public:
    ArmCLProgram(cl_program program, cl_program cpu_program,
            ArmCLContext* armcl_context);
    ~ArmCLProgram();

    cl_program program() const { return program_; }
    cl_program cpu_program() const { return cpu_program_; }

    ArmCLContext* armcl_context() const { return armcl_context_; }

    void BuildCPUProgram();

    void RegisterKernels(ArmCLKernel* kernel);

    void PreprocessingWithSource(const char ** strings,
            const size_t * lengths,
            char ** generated_strings,
            size_t * generated_lengths);

  private:
    void PrepareOpenCLCalls();

    std::vector<ArmCLKernel*> armcl_kernels_;

    cl_program program_;
    cl_program cpu_program_;

    ArmCLContext* armcl_context_;

    //cl_context context_;

    cl_int CL_API_CALL (*clBuildProgram_p_) (cl_program program,
            cl_uint num_devices,
            const cl_device_id* device_list,
            const char* options,
            void (CL_CALLBACK * pfn_notify)(cl_program program,
                void* user_data),
            void* user_data);

};

#endif
